package dev.fyloz.colorrecipesexplorer.rest.account

import dev.fyloz.colorrecipesexplorer.Constants
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeEditUsers
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewUsers
import dev.fyloz.colorrecipesexplorer.dtos.account.UserDto
import dev.fyloz.colorrecipesexplorer.dtos.account.UserSaveDto
import dev.fyloz.colorrecipesexplorer.dtos.account.UserUpdateDto
import dev.fyloz.colorrecipesexplorer.logic.account.UserLogic
import dev.fyloz.colorrecipesexplorer.model.account.Permission
import dev.fyloz.colorrecipesexplorer.rest.created
import dev.fyloz.colorrecipesexplorer.rest.noContent
import dev.fyloz.colorrecipesexplorer.rest.ok
import org.springframework.context.annotation.Profile
import org.springframework.http.MediaType
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.servlet.http.HttpServletRequest
import javax.validation.Valid

@RestController
@RequestMapping(Constants.ControllerPaths.USER)
@Profile("!emergency")
class UserController(private val userLogic: UserLogic) {
    @GetMapping
    @PreAuthorizeViewUsers
    fun getAll() =
        ok(userLogic.getAll())

    @GetMapping("{id}")
    @PreAuthorizeViewUsers
    fun getById(@PathVariable id: Long) =
        ok(userLogic.getById(id))

    @PostMapping
    @PreAuthorizeEditUsers
    fun save(@Valid @RequestBody user: UserSaveDto) =
        created<UserDto>(Constants.ControllerPaths.USER) {
            userLogic.save(user)
        }

    @PutMapping
    @PreAuthorizeEditUsers
    fun update(@Valid @RequestBody user: UserUpdateDto) =
        noContent {
            userLogic.update(user)
        }

    @PutMapping("{id}/password", consumes = [MediaType.TEXT_PLAIN_VALUE])
    @PreAuthorizeEditUsers
    fun updatePassword(@PathVariable id: Long, @RequestBody password: String) =
        noContent {
            userLogic.updatePassword(id, password)
        }

    @PutMapping("{userId}/permissions/{permission}")
    @PreAuthorizeEditUsers
    fun addPermission(
        @PathVariable userId: Long,
        @PathVariable permission: Permission
    ) = noContent {
        userLogic.addPermission(userId, permission)
    }

    @DeleteMapping("{userId}/permissions/{permission}")
    @PreAuthorizeEditUsers
    fun removePermission(
        @PathVariable userId: Long,
        @PathVariable permission: Permission
    ) = noContent {
        userLogic.removePermission(userId, permission)
    }

    @DeleteMapping("{id}")
    @PreAuthorizeEditUsers
    fun deleteById(@PathVariable id: Long) =
        userLogic.deleteById(id)
}

@RestController
@RequestMapping(Constants.ControllerPaths.LOGOUT)
@Profile("!emergency")
class LogoutController(private val userLogic: UserLogic) {
    @GetMapping
    @PreAuthorize("isFullyAuthenticated()")
    fun logout(request: HttpServletRequest) =
        ok {
            userLogic.logout(request)
        }
}
